@@ -47,7 +47,7 @@
 #endif
 
 /* Decimal number made with the formula yymmvv */
-#define ECL_VERSION_NUMBER 210201
+#define ECL_VERSION_NUMBER 230909
 
 /*
  * FEATURES LINKED IN
@@ -140,16 +140,16 @@
  * use autoconf to guess the following values.
  */
 #define ECL_INT_BITS            32
-#define ECL_LONG_BITS           32
-#define ECL_FIXNUM_BITS         32
-#define MOST_POSITIVE_FIXNUM    ((cl_fixnum)536870911)
-#define MOST_NEGATIVE_FIXNUM    ((cl_fixnum)-536870912)
-#define MOST_POSITIVE_FIXNUM_VAL 536870911
-#define MOST_NEGATIVE_FIXNUM_VAL -536870912
+#define ECL_LONG_BITS           64
+#define ECL_FIXNUM_BITS         64
+#define MOST_POSITIVE_FIXNUM    ((cl_fixnum)2305843009213693951L)
+#define MOST_NEGATIVE_FIXNUM    ((cl_fixnum)-2305843009213693952L)
+#define MOST_POSITIVE_FIXNUM_VAL 2305843009213693951L
+#define MOST_NEGATIVE_FIXNUM_VAL -2305843009213693952L
 
-typedef int cl_fixnum;
-typedef unsigned int cl_index;
-typedef unsigned int cl_hashkey;
+typedef long int cl_fixnum;
+typedef unsigned long int cl_index;
+typedef unsigned long int cl_hashkey;
 
 #define ECL_BIGNUM_REGISTER_NUMBER 3
 
@@ -165,8 +165,8 @@ typedef unsigned char ecl_base_char;
  */
 #define ECL_ARRAY_RANK_LIMIT 64 /*  array rank limit            */
 #ifdef GBC_BOEHM
-#define ECL_ARRAY_DIMENSION_LIMIT 536870911
-#define ECL_ARRAY_TOTAL_LIMIT 536870911
+#define ECL_ARRAY_DIMENSION_LIMIT 2305843009213693951L
+#define ECL_ARRAY_TOTAL_LIMIT 2305843009213693951L
 #else
 #define ECL_ARRAY_DIMENSION_LIMIT 16*1024*1024
 #define ECL_ARRAY_TOTAL_LIMIT 16*1024*1024
@@ -232,6 +232,9 @@ typedef unsigned char ecl_base_char;
 /* feenableexcept is available */
 #undef HAVE_FEENABLEEXCEPT
 
+/* wide-strings are available */
+#undef HAVE_WCHAR_H
+
 /*
  * C macros for inlining, denoting probable code paths and other stuff
  * that makes better code. Most of it is GCC specific.
